package gran.model;

import gran.app.adapter.AdapterManager;
import gran.exception.GranException;

import java.io.Serializable;
import java.util.*;


/** Describes custom field (User Defined Field, UDF) */
public class Udf implements Serializable, Comparable {
    public static int STRING = 0;
    public static int FLOAT = 1;
    public static int DATE = 2;
    public static int LIST = 3;
    public static int INTEGER = 4;
    public static int MEMO = 5;


    private String id; //identifier
    private String caption; //persistent
    private Integer order; //persistent
    private String def; //persistent
    private Integer required; //persistent
    private Integer type; //persistent
    private Integer visible; //persistent
    private Integer mail; //persistent
    private Set udflistSet = new HashSet(); //persistent
    private Set udfvalSet = new HashSet(); //persistent
    private Longtext formulaLongtext;

    public Longtext getFormulaLongtext() {
        return formulaLongtext;
    }

    public void setFormulaLongtext(Longtext formulaLongtext) {
        this.formulaLongtext = formulaLongtext;
    }

    private Collection ul;
    private HashMap hm;

    private Udfsource udfsource;

    public Udf(String id, String caption, Integer order, String def, Integer required, Integer type, Integer visible, Integer mail, Longtext formulaLongtext, Udfsource udfsource) {
        this.id = id;
        this.caption = caption;
        this.order = order;
        this.def = def;
        this.required = required;
        this.type = type;
        this.visible = visible;
        this.mail = mail;
        this.udfsource = udfsource;
        this.formulaLongtext = formulaLongtext;
    }

    public Udf() {
    }

    public Udf(String id, Udfsource udfsource) {
        this.id = id;
        this.udfsource = udfsource;
    }

    public String getId() {
        return this.id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getCaption() {
        return this.caption;
    }

    public boolean hasFormula() throws GranException {
        if (formulaLongtext!=null)
        {
            String a = AdapterManager.getInstance().getLongtextAdapterManager().getLongtext(this.formulaLongtext.getId());
            if (a!=null && a.length()>0) return true;
        };
        return false;
    }

    public void setCaption(String caption) {
        this.caption = caption;
    }

    public Integer getOrder() {
        return this.order;
    }

    public void setOrder(Integer order) {
        this.order = order;
    }

    public String getDef() {
        return this.def;
    }

    public void setDef(String def) {
        this.def = def;
    }

    public Integer getRequired() {
        return this.required;
    }

    public void setRequired(Integer required) {
        this.required = required;
    }

    public Integer getType() {
        return this.type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    public Integer getVisible() {
        return this.visible;
    }

    public void setVisible(Integer visible) {
        this.visible = visible;
    }

    public Integer getMail() {
        return this.mail;
    }

    public void setMail(Integer mail) {
        this.mail = mail;
    }

    public Udfsource getUdfsource() {
        return this.udfsource;
    }

    public void setUdfsource(Udfsource udfsource) {
        this.udfsource = udfsource;
    }

    public Set getUdflistSet() {
        return this.udflistSet;
    }

    public void setUdflistSet(Set udflistSet) {
        this.udflistSet = udflistSet;
    }

    public Set getUdfvalSet() {
        return this.udfvalSet;
    }

    public void setUdfvalSet(Set udfvalSet) {
        this.udfvalSet = udfvalSet;
    }


    public Collection getUL() {
        return ul;
    }

    public String getULString() {
        String list = new String("");

        for (Iterator ulit = ul.iterator(); ulit.hasNext();) {
            Udflist ulist = (Udflist) ulit.next();
            list = list + ulist.getVal() + "<BR>";
        }

        return list;
    }


    public void setUL(Collection ul) {
        this.ul = ul;
    }

    public void setHashMap(HashMap h) {
        this.hm = h;
    }

    public HashMap getMap() {
        return this.hm;
    }

    /*
    public int compareTo(Object it) {
        if (caption == null) {
            return 1;
        }
        String other = ((Udf) it).getCaption();
        if (other == null) {
            return -1;
        }
        return this.caption.compareTo(((Udf) it).getCaption());
    }
    */

    public int compareTo(Object it) {

        if (order == null) {
            return 1;
        }
        Integer other = ((Udf) it).getOrder();
        if (other == null) {
            return -1;
        }
        int i= this.order.compareTo(((Udf) it).getOrder());
        if (i==0) return (order+id).compareTo(((Udf) it).getOrder()+((Udf) it).getId());
        else return i;
    }

}
